# !/usr/bin/env python
# encoding: utf-8

from cachetools import TTLCache, cached
from ckan.plugins import toolkit


@cached(cache=TTLCache(maxsize=10, ttl=300))
def get_active_queues():
    jobs = toolkit.get_action('job_list')({'ignore_auth': True}, {})
    pending_queues = set([j['queue'] for j in jobs])
    return len(pending_queues)



#!/usr/bin/env python
# encoding: utf-8
#
# This file is part of ckanext-status
# Created by the Natural History Museum in London, UK

from datetime import datetime as dt

from ckan.plugins import PluginImplementations, toolkit
from ckantools.decorators import action

from ckanext.status.interfaces import IStatus
from ckanext.status.lib.utils import get_active_queues

ignore_missing = toolkit.get_validator('ignore_missing')

status_help = """
A list of status reports from across the system.
"""

status_schema = {'state': [ignore_missing, str]}


@action(status_schema, status_help, get=True)
def status_list(state=None):
    status_reports = []

    pending_queues = get_active_queues()

    status_reports.append(
        {
            'label': toolkit._('Active queues'),
            'value': pending_queues,
            'group': toolkit._('Queues'),
            'help': toolkit._(
                'Number of queues with pending (not actively processing) items'
            ),
            'state': 'good'
            if pending_queues < 2
            else ('ok' if pending_queues < 4 else 'bad'),
        }
    )

    for plugin in PluginImplementations(IStatus):
        status_reports = plugin.modify_status_reports(status_reports)

    if state:
        status_reports = [i for i in status_reports if i['state'] == state]

    return {'reports': status_reports, 'updated_at': dt.now()}



#!/usr/bin/env python
# encoding: utf-8
#
# This file is part of ckanext-status
# Created by the Natural History Museum in London, UK

from ckan.plugins import interfaces


class IStatus(interfaces.Interface):
    """
    This interface allows extensions to modify and add status reports for various
    systems.
    """

    def modify_status_reports(self, status_reports):
        """
        Modify the list of items rendered on the status page.

        :param status_reports: items to present on the status page
        :returns: the modified list
        """
        return status_reports



#!/usr/bin/env python
# encoding: utf-8
#
# This file is part of ckanext-status
# Created by the Natural History Museum in London, UK

from ckantools.decorators import auth
from ckantools.vars import auth_valid


@auth(anon=True)
def status_list(context, data_dict):
    """
    Allow for everyone.
    """
    return auth_valid



#!/usr/bin/env python
# encoding: utf-8
#
# This file is part of ckanext-status
# Created by the Natural History Museum in London, UK

import itertools

from ckan.plugins import toolkit
from flask import Blueprint

blueprint = Blueprint(name='status', import_name=__name__, url_prefix='/status')


@blueprint.route('/')
def index():
    """
    Renders an HTML page showing status items.
    """
    status_response = toolkit.get_action('status_list')({}, {})

    grouped_reports = {
        k: list(v)
        for k, v in itertools.groupby(
            sorted(status_response['reports'], key=lambda x: x.get('group', '_root')),
            key=lambda x: x.get('group', '_root'),
        )
    }

    return toolkit.render(
        'status/index.html',
        extra_vars={
            'reports': grouped_reports,
            'updated_at': status_response['updated_at'],
        },
    )
